@@ -907,7 +907,7 @@ generic_get_memory_info( PAPI_hw_info_t *hw_info )
 	char filename[BUFSIZ],type_string[BUFSIZ];
 	struct dirent *d;
 	int max_level=0;
-	int level_count=0,last_level=-1,level_index=0;
+	int level_count,level_index;
 
 	PAPI_mh_level_t *L = hw_info->mem_hierarchy.level;
 
@@ -920,6 +920,12 @@ generic_get_memory_info( PAPI_hw_info_t *hw_info )
 		goto unrecoverable_error;
 	}
 
+	for (level_index=0; level_index < PAPI_MAX_MEM_HIERARCHY_LEVELS; ++level_index) {
+		for (level_count = 0; level_count < PAPI_MH_MAX_LEVELS; ++level_count) {
+			L[level_index].cache[level_count].type = PAPI_MH_TYPE_EMPTY;
+		}
+	}
+
 	while(1) {
 		d = readdir(dir);
 		if (d==NULL) break;
@@ -950,11 +956,12 @@ generic_get_memory_info( PAPI_hw_info_t *hw_info )
 		/* Index arrays from 0 */
 		level_index=level-1;
 
-		if (level!=last_level) {
-			level_count=0;
-			last_level=level;
-		} else {
+		level_count = 0;
+		while (L[level_index].cache[level_count].type != PAPI_MH_TYPE_EMPTY) {
 			level_count++;
+			if (level_count>=PAPI_MH_MAX_LEVELS) {
+				break;
+			}
 		}
 
 		if (level_count>=PAPI_MH_MAX_LEVELS) {
@@ -1093,6 +1100,7 @@ generic_get_memory_info( PAPI_hw_info_t *hw_info )
 		}
 	}
 
+   closedir(dir);
 	hw_info->mem_hierarchy.levels = max_level;
 
 	return 0;
@@ -1102,6 +1110,7 @@ unrecoverable_error:
 	/* Just say we have no cache */
 	hw_info->mem_hierarchy.levels = 0;
 
+   closedir(dir);
 	return 0;
 }
 
@@ -1217,7 +1226,8 @@ _linux_update_shlib_info( papi_mdi_t *mdi )
 					t_index++;
 					tmp[t_index - 1].text_start = ( caddr_t ) begin;
 					tmp[t_index - 1].text_end = ( caddr_t ) ( begin + size );
-					strncpy( tmp[t_index - 1].name, mapname, PAPI_MAX_STR_LEN );
+					strncpy( tmp[t_index - 1].name, mapname, PAPI_HUGE_STR_LEN );
+               tmp[t_index - 1].name[PAPI_HUGE_STR_LEN-1]=0;
 				}
 			} else if ( ( perm[0] == 'r' ) && ( perm[1] == 'w' ) &&
 						( inode != 0 ) ) {
